from .. import Provider as BaseProvider


# Reference:
# https://th.wikipedia.org/wiki/หมวดหมู่:บุคคลแบ่งตามอาชีพ
# on 2018-10-16
class Provider(BaseProvider):
    jobs = [
        'นักกฎหมาย',
        'กวี',
        'นักการทูต',
        'นักการเมือง',
        'นักการศึกษา',
        'นักกีฬา',
        'นักการกุศล',
        'เกษตรกร',
        'นักเขียน',
        'ข้าราชการ',
        'นักคณิตศาสตร์',
        'คนขับรถแท็กซี่',
        'โฆษก',
        'จ๊อกกี้',
        'นักจัดรายการวิทยุ',
        'จารชน',
        'จิตรกร',
        'นักจิตวิทยา',
        'เจ้าหน้าทีรักษาความปลอดภัย',
        'เจ้าหน้าที่รัฐบาล',
        'ช่างทำเครื่องดนตรี',
        'ช่างทำผม',
        'นักชีววิทยา',
        'นักดนตรี',
        'นักดาราศาสตร์',
        'นักแต่งเพลง',
        'ตำรวจ',
        'นักถ่ายภาพ',
        'ทนายความ',
        'ทหารบก',
        'นักธุรกิจ',
        'นักเคลื่อนไหว',
        'นักบวช',
        'นักบิน',
        'นักบินอวกาศ',
        'นักประชาสัมพันธ์',
        'นักผจญภัย',
        'นักสะสมศิลปะ',
        'นักแสดง',
        'นักหนังสือพิมพ์',
        'นางงาม',
        'นางแบบ',
        'นายแบบ',
        'บรรณาธิการ',
        'นักโบราณคดี',
        'นักแปล',
        'นักประดิษฐ์',
        'นักประวัติศาสตร์',
        'นักปรัชญา',
        'ผู้กำกับ',
        'ผู้กำกับภาพยนตร์',
        'ผู้กำกับละครโทรทัศน์',
        'ผู้จัดพิมพ์',
        'นักพจนานุกรม',
        'แพทย์',
        'นักพากย์',
        'พิธีกร',
        'นักโภชนาการ',
        'นักภาษาศาสตร์',
        'เภสัชกร',
        'มัคคุเทศก์',
        'นักมายากล',
        'นักวาดการ์ตูน',
        'นักวิทยาศาสตร์',
        'วิศวกร',
        'วีเจ',
        'นักเศรษฐศาสตร์',
        'ศิลปิน',
        'สถาปนิก',
        'นักสังคมวิทยา',
        'นักสังคมศาสตร์',
        'นักสัตววิทยา',
        'นักสำรวจ',
        'นักสืบ',
        'นักอนุรักษ์ธรรมชาติ',
        'นักออกแบบ',
        'อัยการ',
        'โปรแกรมเมอร์',
    ]



from django.forms.models import inlineformset_factory

from scarlet.cms import bundles, forms, options, views, renders, actions
from scarlet.cms.sites import site

from .models import Post, PostImage, Comment, Category, Tag, Author, DummyModel
from .views import PostsListView
from .forms import EditAuthorForm

postimages_formset = forms.LazyFormSetFactory(
    inlineformset_factory, Post, PostImage, can_order=True
)


class CloneAction(actions.ActionView):
    short_description = "Clone selected posts."

    def process_action(self, request, queryset):
        for obj in queryset:
            post = Post.objects.create(
                date=obj.date,
                title=obj.title,
                slug=obj.slug,
                body=obj.body,
                author=obj.author,
                category=obj.category,
                tag=obj.tags,
                keywords=obj.keywords,
                description=obj.description,
            )
        num = queryset.count()
        msg = f"{num} object{'s' if num == 1 else ''} have been cloned."
        self.write_message(message=msg)
        return self.render(request, redirect_url=self.get_done_url(), message=msg)


class CloneCommentAction(actions.ActionView):
    short_description = "Clone selected comments."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.renders["object_header"] = renders.RenderString(
            template=self.object_header_tmpl
        )

    def process_action(self, request, queryset):
        for obj in queryset:
            comment = Comment.objects.create(
                post=obj.post, name=obj.name, text=obj.text
            )
        num = queryset.count()
        msg = f"{num} object{'' if num == 1 else 's'} have been cloned."
        self.write_message(message=msg)
        return self.render(request, redirect_url=self.get_done_url(), message=msg)


class DummyActionView(actions.ActionView):
    def process_action(self, request, queryset):
        queryset.update(title="Dummy")
        count = queryset.count()
        msg = (
            f'{count} object{"" if count == 1 else "s"} '
            'have been changed to "Dummy".'
        )
        self.write_message(message=msg)
        return self.render(request, redirect_url=self.get_done_url(), message=msg)


class SomethingAction(actions.ActionView):
    short_description = "It does something."
    confirmation_message = "Sure you want to do something to the following objects?"

    def process_action(self, request, queryset):
        queryset.update(name="Something")


class PostImageBundle(bundles.Bundle):
    navigation = bundles.PARENT

    class Meta(options.Orderable):
        model = PostImage


class CategoryBundle(bundles.Bundle):
    navigation = bundles.PARENT
    main = views.ListView(
        paginate_by=1, display_fields=("category",), change_fields=("category",)
    )

    class Meta:
        model = Category
        primary_model_bundle = True


class TagBundle(bundles.Bundle):
    navigation = bundles.PARENT
    main = views.ListView(
        paginate_by=1, display_fields=("tag",), change_fields=("tag",)
    )

    class Meta:
        model = Tag
        primary_model_bundle = True


class CommentBundle(bundles.ParentVersionedBundle, bundles.VersionMixin):
    navigation = bundles.PARENT
    object_view = bundles.PARENT
    clone = CloneCommentAction()
    main = views.ListView(display_fields=("name", "text"))
    something = SomethingAction()

    class Meta:
        model = Comment
        parent_field = "post"
        action_views = ["delete", "clone", "something", "publish", "unpublish"]


class AuthorBundle(bundles.Bundle):
    navigation = bundles.PARENT
    edit = views.FormView(form_class=EditAuthorForm)
    main = views.ListView(filter_form=forms.search_form("name", "bio",))

    class Meta:
        model = Author


class DummyAliasBundle(bundles.Bundle):
    dummy_edit = views.FormView()
    edit = bundles.URLAlias(alias_to="dummy_edit")

    class Meta:
        model = DummyModel
        item_views = ("dummy_edit", "delete")


class DummyRedirectorBundle(bundles.Bundle):
    edit = views.FormView(redirect_to_view="edit")

    class Meta:
        model = DummyModel
        item_views = ("edit", "delete")


DEFAULT_FIELDS = (
    ("Post", {"fields": ("date", "title", "body", "author", "category", "tags")}),
)


class PostDeleteView(actions.DeleteActionView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.renders["object_header_bare"] = renders.RenderString(
            template="cms/object_header_bare.html"
        )


class BlogEditBundle(bundles.VersionedObjectOnlyBundle):
    navigation = (
        ("main", "Post Data"),
        ("comments", "Comments"),
        ("seo", "Page SEO"),
        ("delete", "Delete"),
        ("publish", "Publish"),
        ("unpublish", "Unpublish"),
    )

    main = views.FormView(
        redirect_to_view=None,
        formsets={"Images": postimages_formset},
        fieldsets=DEFAULT_FIELDS,
    )

    delete = PostDeleteView()
    edit = bundles.PARENT
    comments = CommentBundle.as_subbundle(name="comments", title="Comments",)
    seo = views.FormView(
        redirect_to_view=None,
        cancel_view=None,
        fieldsets=(("SEO", {"fields": ("keywords", "description",)}),),
    )

    class Meta:
        model = Post


class PostAddView(views.FormView):
    def get(self, request, *args, **kwargs):
        if not self.model.category.get_query_set():
            self.write_message(
                status=30,
                message="Warning! You need at least one Category to insert new Posts.",
            )
        if not self.model.author.get_query_set():
            self.write_message(
                status=30,
                message="Warning! You need at least one Author to insert new Posts.",
            )
        return super().get(request, *args, **kwargs)


class BlogBundle(bundles.DelegatedObjectBundle):
    dashboard = (
        ("main",),
        ("author",),
        ("dummy_alias",),
        ("dummy_redirector",),
        ("category",),
        ("tag",),
    )

    main = views.ListView(display_fields=("title", "author"))
    delete = actions.DeleteActionView()
    add = PostAddView(fieldsets=DEFAULT_FIELDS)
    edit = BlogEditBundle.as_subbundle(name="post", title="Post")
    author = AuthorBundle.as_subbundle(name="author", title="Author")
    dummy_alias = DummyAliasBundle.as_subbundle(name="dummy_alias", title="Dummy Alias")
    dummy_redirector = DummyRedirectorBundle.as_subbundle(
        name="dummy_redirector", title="Dummy Redirect"
    )
    category = CategoryBundle.as_subbundle(name="category", title="Category")
    tag = TagBundle.as_subbundle(name="tag", title="Tag")
    preview = views.PreviewWrapper(preview_view=PostsListView, pass_through_kwarg=None)
    change = DummyActionView(short_description="Change names to 'Dummy'")
    clone = CloneAction()

    class Meta:
        model = Post
        primary_model_bundle = True
        item_views = list(options.VersionMeta.item_views) + ["preview"]
        action_views = ["change", "delete", "clone"]


class BigAuthorBundle(bundles.DelegatedObjectBundle):
    dashboard = (("author",),)

    author = AuthorBundle.as_subbundle(name="author", title="Author")


class DummySingletonBundle(bundles.SingletonBundle):
    dashboard = (("main",),)
    edit = views.SingletonFormView()
    main = edit

    class Meta:
        model = DummyModel
        primary_model_bundle = False


site.register("dummy", BlogBundle(name="dummy"), order=10)
site.register("authoronly", BigAuthorBundle(name="authoronly"), order=10)
site.register("dummysingleton", DummySingletonBundle(name="dummysingleton"), order=10)



from django.utils.safestring import mark_safe

try:
    from ..cms import bundles, cms_bundles, views
    from ..cms.sites import site
except ValueError:
    from cms import site, bundles, views, cms_bundles

from . import forms, get_asset_model
from .views import AssetFormView, AssetListView


def preview(obj):
    if obj.type in [obj.IMAGE, obj.SVG_IMAGE]:
        thumbnail = obj.file.admin_url()
        if thumbnail:
            return mark_safe('<img src="{0}" />'.format(thumbnail))

    return ""


class AssetBundle(bundles.Bundle):

    main = AssetListView(
        display_fields=(preview, "title", "user_filename", "modified", "type")
    )
    add = AssetFormView(force_add=True, form_class=forms.UploadAssetForm)
    edit = AssetFormView(form_class=forms.UpdateAssetCropForm)

    class Meta:
        item_views = ("edit", "delete")
        primary_model_bundle = True
        model = get_asset_model()


class EmbedView(cms_bundles.EmbedView):
    def get(self, request, *args, **kwargs):
        tags = request.GET.get("tags")
        bundle = self.bundle.admin_site.get_bundle_for_model(get_asset_model())
        api_link = ""
        if bundle:
            api_link = bundle.get_view_url("main", request.user)
            if api_link:
                api_link = "{0}?type=choices".format(api_link)

        return self.render(request, tags=tags, api_link=api_link)


class WYSIWYG(cms_bundles.WYSIWYG):
    main = EmbedView(default_template="cms/insert_media.html")


site.unregister("wysiwyg")
site.register("wysiwyg", WYSIWYG(name="wysiwyg"), 21)
site.register("assets", AssetBundle(name="assets"), 20)



from django.apps import AppConfig

from .errors import BAD_CONFIG_ERROR


def webpack_cfg_check(*args, **kwargs):
    '''Test if config is compatible or not'''
    from django.conf import settings

    check_failed = False
    user_config = getattr(settings, 'WEBPACK_LOADER', {})
    try:
        user_config = [dict({}, **cfg) for cfg in user_config.values()]
    except TypeError:
        check_failed = True

    errors = []
    if check_failed:
        errors.append(BAD_CONFIG_ERROR)
    return errors


class WebpackLoaderConfig(AppConfig):
    name = 'webpack_loader'
    verbose_name = "Webpack Loader"

    def ready(self):
        from django.core.checks import register, Tags
        register(Tags.compatibility)(webpack_cfg_check)



# Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt

"""
Imports that we need at runtime, but might not be present.

When importing one of these modules, always do it in the function where you
need the module.  Some tests will need to remove the module.  If you import
it at the top level of your module, then the test won't be able to simulate
the module being unimportable.

The import will always succeed, but the value will be None if the module is
unavailable.

Bad::

    # MyModule.py
    from coverage.optional import unsure

    def use_unsure():
        unsure.something()

Good::

    # MyModule.py

    def use_unsure():
        from coverage.optional import unsure
        if unsure is None:
            raise Exception("Module unsure isn't available!")

        unsure.something()

"""

import contextlib

# This file's purpose is to provide modules to be imported from here.
# pylint: disable=unused-import

# TOML support is an install-time extra option.
try:
    import toml
except ImportError:         # pragma: not covered
    toml = None


@contextlib.contextmanager
def without(modname):
    """Hide a module for testing.

    Use this in a test function to make an optional module unavailable during
    the test::

        with coverage.optional.without('toml'):
            use_toml_somehow()

    Arguments:
        modname (str): the name of a module importable from
            `coverage.optional`.

    """
    real_module = globals()[modname]
    try:
        globals()[modname] = None
        yield
    finally:
        globals()[modname] = real_module



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    def day_of_week(self):
        day = self.date('%w')
        DAY_NAMES = {
            "0": "Dimanche",
            "1": "Lundi",
            "2": "Mardi",
            "3": "Mercredi",
            "4": "Jeudi",
            "5": "Vendredi",
            "6": "Samedi",
        }
        return DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        MONTH_NAMES = {
            "01": "Janvier",
            "02": "Février",
            "03": "Mars",
            "04": "Avril",
            "05": "Mai",
            "06": "Juin",
            "07": "Juillet",
            "08": "Août",
            "09": "Septembre",
            "10": "Octobre",
            "11": "Novembre",
            "12": "Décembre",
        }
        return MONTH_NAMES[month]
